struct inh_def {

	int 	conid     ; /* config id #               */
	short	icocd     ; /* config int code           */
	int 	traid     ; /* track id #                */
	int 	inhid     ; /* integration id #          */
	int 	ints      ; /* integration #             */
	short	itq       ; /* tuning int code           */
	float	az        ; /* azimuth                   */
	float	el        ; /* elevation                 */
	float	ha        ; /* hour angle                */
	short	iut       ; /* ut int code               */
	short	iref_time ; /* ref_time int code         */
	double	dhrs      ; /* hrs from ref_time         */
	float	vc        ; /* vcorr for vctype          */
	short	ivctype   ; /* vctype int code           */
	double	sx        ; /* x vec. for bsl.           */
	double	sy        ; /* y vec. for bsl.           */
	double	sz        ; /* z vec. for bsl.           */
	float	rinteg    ; /* actual int time           */
	int 	proid     ; /* project id #              */
	int 	souid     ; /* source id #               */
	short	isource   ; /* source int code           */
	short	ipos      ; /* position int code         */
	float	offx      ; /* offset in x               */
	float	offy      ; /* offset in y               */
	short	iofftype  ; /* offset int code           */
	short	ira       ; /* ra int code               */
	short	idec      ; /* dec int code              */
	double	rar       ; /* ra (radians)              */
	double	decr      ; /* declination (radians)     */
	float	epoch     ; /* epoch for coord.          */
	float	sflux     ; /* flux                      */
	float	size      ; /* source size               */

};

/* inh is 132 bytes if short=2 int=4 float=4 double=8 */

struct blh_def {

	int 	blhid     ; /*	proj. baseline id #       */
	int 	inhid     ; /*	integration id #          */
	short	isb       ; /*	sideband int code         */
	short	ipol      ; /*	polarization int code     */
	float	pa        ; /*	polarization angle        */
	short	iaq       ; /*	amp qual int code         */
	short	ibq       ; /*	baseline qual int code    */
	short	icq       ; /*	coherence qual int code   */
	short	ioq       ; /*	offset qual int code      */
	short	irec      ; /*	receiver int code         */
	short	iifc      ; /*	if channel int code       */
	float	u         ; /*	u coord for bsl (klambda) */
	float	v         ; /*	v coord. for bsl          */
	float	w         ; /*	w coord. for bsl          */
	float	prbl      ; /*	projected baseline        */
	float	angres    ; /*	baseline resolution       */
	float	vis       ; /*	fractional visibility     */
	float	coh       ; /*	coherence estimate        */
	float	sigcoh    ; /*	sigma on coh.             */
	float	csnr      ; /*	cont. SNR                 */
	float	vflux     ; /*	visibility flux           */
	float	cnoise    ; /*	continuum noise           */
	double	avedhrs   ; /*	hrs offset from ref-time  */
	float	ampave    ; /*	ave continuum amp         */
	float	phaave    ; /*	ave continuum phase       */
	float	tpvar     ; /*	total power stability     */
	int 	blsid     ; /*	physical baseline id #    */
	short	itel1     ; /*	tel 1 int code            */
	short	itel2     ; /*	tel 2 int code            */
	short	iblcd     ; /*	baseline int code         */
	float	ble       ; /*	bsl east vector           */
	float	bln       ; /*	bsl north vector          */
	float	blu       ; /*	bsl up vector             */
	int 	soid      ; /*	bsl soln id #             */

};

/* the size of blh_def is 118 bytes */

struct sph_def {

	int 	sphid     ; /*  spectrum id #             */
	int 	blhid     ; /*  proj. baseline id #       */
	int 	inhid     ; /*  integration id #          */
	short	igq       ; /*  gain qual int code        */
	short	ipq       ; /*  passband qual int code    */
	short	iband     ; /*  spectral band int code    */
	short	ipstate   ; /*  pol state int code        */
	float	tau0      ; /*  tau at zenith             */
	double	vel       ; /*  velocity (vctype) (km/s)  */
	float	vres      ; /*  velocity res.             */
	short	ivtype    ; /*  velocity type int code    */
	double	fsky      ; /*  center sky freq.          */
	float	fres      ; /*  frequency res. (MHz)      */
	float	tssb      ; /*  tsys (ssb)                */
	float	integ     ; /*  integration time          */
	float	wt        ; /*  weight (sec/tssb**2)      */
	short	itaper    ; /*  spectrum taper int code   */
	float	snoise    ; /*  th. noise in spectrum     */
	short	nch       ; /*  # channels in spectrum    */
	short	nrec      ; /*  # of records w/i inh#     */
	int 	dataoff   ; /*  byte offset for data      */
	int 	linid     ; /*  spectral line id #        */
	short	itrans    ; /*  transition int code       */
	double	rfreq     ; /*  rest frequency (GHz)      */
	short	pasid     ; /*  passband fn id #          */
	short	gaiidamp  ; /*  amp gain fn id #          */
	short	gaiidpha  ; /*  pha gain fn id #          */
	short	flcid     ; /*  flux cal id #             */
	short	atmid     ; /*  atmospheric coh id #      */

};

/* the size of sph_def is 100 bytes */

struct codeh_def {

	char v_name[12]	; /* label 			*/
	short  icode	; /* index for a code word	*/
	char code[26]	; /* the code word		*/
	short ncode	; /* # chars in code word	*/

};

/* the size of codeh is 42 bytes char=1 short=2 */


/*
;   In the database archive, each record of data consists of 4 header
;   numbers followed by a variable number of complex channels.
;   The beginning of the record data stream is given by dataOFF from SPH.
;   Record format:
;      ITEM     SIZE OFF
;     integ      i*2   0       Integration time in .1secs
;     toff       i*2   2       Time offset from INH ut in secs
;     dnoise     i*4   4       Noise estimate in milliJanskies (will be read 
				by the data processing program as 2 shorts and
				then reconstituted as a long)
;     scale      i*2   8       Scale exponent for data
;     ch[1].real i*2  10       Signed, scaled, short (2-byte integer) data
;     ch[1].imag i*2
;     ch[2].real i*2
;     ch[2].imag i*2
;     ...
;
;   The total size of each record is 10+nch*4 bytes.
;   Each data point is stored as a 2 byte (short) integer. The restored data
;   (in Janskies) are computed as follows:
;                 restored_data = 2**scale * short_data
;
;   The noise estimate (dnoise) is stored as a 4byte integer in millijanskys.
*/

struct sch_def {
	int inhid	;  /* integration id # */
	char form[4]	;  /* indicates the format of the data, always I2-C */
	int nbyt	;  /* the number of bytes in one integration of data */
	int nbyt_pack	;  /* ? */
	short *packdata	;  /* integer array containing the data in the format above	*/
};

/* 
The dimension of the packdata integer array:  
number of points in one record		=	5 + 2* number of channels	
number of records in each spectrum	=	nrec (always 1 for our data for now)
number of spectra in each baseline	=	numberBands (in this code)
number of baselines in each integration	=	numberBaselines (in this code)

the number of sph header structures written is numberBands*numberBaselines, 
and this is called the number of spectra in the idl programs.
*/

struct rh_def {
	int   noiseEst   ;  /* noise estimate for record in (mJy)                        */
	short integTime  ;  /* The integration time of the record in 0.1 secs            */
	short timeAfter  ;  /* Time offset from INH UT time in secs                */                   
};


struct ant_def{
	int antennaNumber;
	int padNumber;
	int antennaStatus;  /* Antenna is ON or OFF LINE */
	int trackStatus;    /* Track is running or not */
	int commStatus;     /* Data for this integration is valid or not */

	int 	inhid     ; /* integration id #          */
	int 	ints      ; /* integration #             */
	double	dhrs      ; /* hrs from ref_time         */
	double	ha        ; /* hour angle                */
	double	lst       ; /* lst                       */

        double pmdaz ;                       /* pointing model correction */
        double pmdel ;
        double tiltx ;
        double tilty ;
 
        double actual_az ;
        double actual_el ;
        double azoff ;
        double eloff ;
        double az_tracking_error ;
        double el_tracking_error ;
        double refraction ;
        double chopper_x ;
        double chopper_y ;
        double chopper_z ;
        double chopper_angle ;
 
        double tsys ;
        double ambient_load_temperature ;

};
